var user_id;
var customerId;
var currentClientCompliances = null;
var compliances;
var complianceType;
var projectId = null;
var clientname;
var userfullname;


document.addEventListener('DOMContentLoaded', function() {
    // Set initial display state for dropdowns
    document.getElementById("documentsTreeContainer").style.display = 'none';
    document.getElementById("recordsTreeContainer").style.display = 'none';
    customerId = new URLSearchParams(window.location.search).get('customerId');
    if (customerId) {
    console.log(customerId)
        fetchClientData(customerId);
    }
    fetchUserIdFromSession();
    setupTreeButtons();
    const startGapAnalysisButton = document.getElementById('startGapAnalysisButton');
    if (startGapAnalysisButton) {
        startGapAnalysisButton.addEventListener('click', function() {
            console.log("Button click - Customer ID:", customerId);
            if (currentClientCompliances && customerId) {
                createGapAnalysis();
            } else {
                console.error('No compliance data available for the client or customer ID is missing');
            }
        });
    }
});

function fetchUserIdFromSession() {
    // Make a GET request to a server endpoint that returns the session data
    fetch('/get-user-id', {
        method: 'GET',
        credentials: 'include',  // Include credentials (cookies) in the request
    })
    .then(response => response.json())
    .then(data => {
        // Check if 'user_id' is available in the session data
        if (data.user_id !== undefined) {
            user_id = data.user_id;
            console.log('User ID:', user_id);

            // Fetch user data using 'user_id'
            fetchUserData(user_id);
        } else {
            console.error('User is not authenticated or session data is missing.');
        }
    })
    .catch(error => console.error('Error fetching user session data:', error));
}

function fetchUserData(user_Id) {
    const apiUrl = `/api/user-data/${user_Id}`; // Adjust the URL as needed

    // Make a GET request to the API endpoint
    fetch(apiUrl)
        .then(response => {
            if (!response.ok) {
                throw new Error('Network response was not ok');
            }
            return response.json();
        })
        .then(data => {
            // Creating a UserData object with the response data
            const userData = {
                first_name: data.first_name,
                last_name: data.last_name,
                last_login: data.last_login,
            };
            userfullname = userData.first_name + ' ' + userData.last_name;
            updateUserUI(userData);
            console.log(userData);
        })
        .catch(error => {
            // Handle errors that occurred during the fetch request
            console.error('Fetch error:', error);
        });
}

function fetchClientData(customerId) {
    const parsedCustomerId = parseInt(customerId, 10);

    fetch('/api/clients')
        .then(response => response.json())
        .then(clients => {
            const client = clients.find(c => c.id === parsedCustomerId);

            if (client) {
            clientname = client.name;
            populateClientData(client);
            console.log("Client data:", client);
            } else {
                console.error('Client not found for ID:', parsedCustomerId);
            }
        })
        .catch(error => console.error('Error fetching client data:', error));
}

function updateUserUI(userData) {
    // Update the UI with the user data
    document.getElementById('firstName').textContent = userData.first_name;
    document.getElementById('lastName').textContent = userData.last_name;
}

function populateClientData(client) {
    document.getElementById('clientName').textContent = client.name || 'N/A';
    document.getElementById('clientCompliances').textContent = client.compliances || 'N/A';
    document.getElementById('currentYearPhase').textContent = client.preperation_phase || client.type || 'N/A';
    currentClientCompliances = client.compliances;
    console.log("Fetched compliances after setting:", currentClientCompliances);
}

function startComplianceProject(user_id, customerId, complianceType) {
    console.log("Received complianceType in startComplianceProject:", complianceType);
    // Ensure complianceType is defined
    if (!complianceType) {
        console.error('Compliance type is undefined');
        return;
    }

    projectName = prompt("Please enter a friendly name for your Compliance Project:");
    if (projectName === null || projectName.trim() === "") {
        alert("Compliance project creation canceled.");
        return;
    }

    var backendName = complianceType + "_" + new Date().toISOString();

    var complianceProjectData = {
        user_id: user_id, // Assuming this is the logged-in user's ID
        friendly_name: projectName.trim(),
        backend_name: backendName,
        compliance_type: complianceType,
        customerId: customerId // Ensure this is the client's ID you want to associate with the project
    };

    console.log('Sending data to /start-compliance-project:', complianceProjectData);

    // Make a POST request to start the compliance project
    fetch('/start-compliance-project', {
        method: 'POST',
        credentials: 'include',
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify(complianceProjectData)
    })
    .then(response => {
        if (!response.ok) {
            throw new Error('Network response was not ok: ' + response.statusText);
        }
        return response.json();
    })
    .then(data => {
        console.log('Compliance project started successfully:', data);
        alert('Compliance project started successfully. Please check the dashboard for more details.');
    })
    .catch(error => {
        console.error('Error starting compliance project:', error);
        alert('An error occurred while starting the compliance project.');
    });
}

function createGapAnalysis() {
    if (!currentClientCompliances) {
        console.error('No compliance data available for the client');
        return;
    }

    const complianceList = currentClientCompliances.split(',').map(item => item.trim());

    if (complianceList.length === 1) {
        sendToGapAnalysis(customerId, complianceList[0]);
    } else {
        createComplianceSelection(customerId, complianceList);
    }
}

function createComplianceSelection(customerId, complianceList) {
    const selectionContainer = document.createElement('div');
    selectionContainer.id = 'complianceSelection';
    selectionContainer.innerHTML = '<h3>Gap Analysis</h3>';

    complianceList.forEach(complianceType => {
        const button = document.createElement('button');
        button.textContent = complianceType;
        button.onclick = () => sendToGapAnalysis(customerId, complianceType);
        selectionContainer.appendChild(button);
    });

    // Append the selection container to the body or a specific element on your page
    document.body.appendChild(selectionContainer);
}

function sendToGapAnalysis(customerId, complianceType) {
    console.log('Sending to Gap Analysis for customer:', customerId, 'with compliance:', complianceType);
    if (!complianceType) {
        console.error('Compliance type is undefined in sendToGapAnalysis');
        return;
    }
    startComplianceProject(user_id, customerId, complianceType);
}
function openGapAnalysisDoc() {
    fetch(`/api/compliance-projects/?customerId=${customerId}`)
        .then(response => response.json())
        .then(data => {
            if (data.length === 1) {
                // Only one project, so perform the action directly
                const selectedProjectName = data[0].projectName;
                const complianceType = data[0].complianceType;
                window.location.href = `/gap-analysis-doc?projectName=${encodeURIComponent(selectedProjectName)}&complianceType=${encodeURIComponent(complianceType)}`;
            } else if (data && data.length > 0) {
                const projectNames = data.map((project, index) => `${index + 1}. ${project.projectName}`);
                const projectSelectionPrompt = projectNames.join('\n');
                const selectedProjectNumber = prompt(`Please select a project:\n${projectSelectionPrompt}`);

                if (selectedProjectNumber) {
                    const selectedProjectIndex = parseInt(selectedProjectNumber, 10) - 1;
                    if (selectedProjectIndex >= 0 && selectedProjectIndex < projectNames.length) {
                        const selectedProjectName = projectNames[selectedProjectIndex].split('. ')[1];
                        console.log('Selected project:', selectedProjectName);

                        window.location.href = `/gap-analysis-doc?projectName=${encodeURIComponent(selectedProjectName)}&complianceType=${encodeURIComponent(data[selectedProjectIndex].complianceType)}`;
                    } else {
                        alert('Invalid selection. Please try again.');
                    }
                }
            } else {
                alert('No projects found for this customer.');
            }
        })
        .catch(error => {
            console.error('Error fetching compliance projects:', error);
            alert('An error occurred while fetching compliance projects.');
        });
}


function openMainDashboard() {
    var apiUrl = '/fetch-customer-projects?customer_id=' + customerId;

    fetch(apiUrl)
    .then(response => {
        if (!response.ok) {
            throw new Error('Network response was not ok');
        }
        return response.json();
    })
    .then(data => {
        if (data.projects && data.projects.length > 0) {
            var projectSelectionMessage = 'Please Enter the number of the Project related to the  Project name you would like:\n';
            data.projects.forEach((project, index) => {
                projectSelectionMessage += (index + 1) + '. ' + project.project_name + ' (ID: ' + project.project_id + ')\n';
            });

            var selectedProjectNumber = prompt(projectSelectionMessage);
            var selectedProjectIndex = parseInt(selectedProjectNumber, 10) - 1;

            if (selectedProjectIndex >= 0 && selectedProjectIndex < data.projects.length) {
                var selectedProject = data.projects[selectedProjectIndex];
                projectId = selectedProject.project_id;
                complianceType = selectedProject.compliance_type;

                // Redirect to the project-dashboard with projectId and complianceType
                window.location.href = '/project-dashboard?project_id=' + encodeURIComponent(projectId) + '&complianceType=' + encodeURIComponent(complianceType);
            } else {
                alert('Invalid project number');
            }
        } else {
            alert('No projects found for this customer.');
        }
    })
    .catch(error => {
        console.error('There has been a problem with your fetch operation:', error);
        alert('Error fetching projects');
    });
}

function viewComplianceProjects() {
    // Redirect to the /view-compliance-projects endpoint
    window.location.href = '/view-compliance-projects';
}

function opensoa(complianceType) {
    if (!customerId) {
        console.error('Customer ID is not defined');
        return;
    }

    var apiUrl = '/fetch-customer-projects?customer_id=' + encodeURIComponent(customerId);

    fetch(apiUrl)
        .then(response => {
            if (!response.ok) {
                throw new Error('Network response was not ok');
            }
            return response.json();
        })
        .then(data => {
            if (data.projects && data.projects.length === 1) {
                // Only one project, so perform the action directly
                var selectedProject = data.projects[0];
                var projectId = selectedProject.project_id;

                // Construct the URL with query parameters
                var url = "/SOA?complianceType=" + encodeURIComponent(complianceType) + "&projectId=" + encodeURIComponent(projectId);
                // Redirect the user to the constructed URL
                window.location.href = url;
            } else if (data.projects && data.projects.length > 0) {
                var projectSelectionMessage = 'Please Enter the number of the Project related to the  Project name you would like to see the SOA:\n';
                data.projects.forEach((project, index) => {
                    projectSelectionMessage += (index + 1) + '. ' + project.project_name + ' (ID: ' + project.project_id + ')\n';
                });

                var selectedProjectNumber = prompt(projectSelectionMessage);
                var selectedProjectIndex = parseInt(selectedProjectNumber, 10) - 1;

                if (selectedProjectIndex >= 0 && selectedProjectIndex < data.projects.length) {
                    var selectedProject = data.projects[selectedProjectIndex];
                    var projectId = selectedProject.project_id;

                    // Construct the URL with query parameters
                    var url = "/SOA?complianceType=" + encodeURIComponent(complianceType) + "&projectId=" + encodeURIComponent(projectId);
                    // Redirect the user to the constructed URL
                    window.location.href = url;
                } else {
                    alert('Invalid project number');
                }
            } else {
                alert('No projects found for this customer.');
            }
        })
        .catch(error => {
            console.error('There has been a problem with your fetch operation:', error);
            alert('Error fetching projects');
        });
}


function openIA() {
    var apiUrl = '/fetch-customer-projects?customer_id=' + encodeURIComponent(customerId);

    fetch(apiUrl)
        .then(response => response.json())
        .then(data => {
            if (data.projects && data.projects.length === 1) {
                // Only one project, so perform the action directly
                var selectedProject = data.projects[0];
                checkIARecord(selectedProject.project_id, selectedProject.project_name);
            } else if (data.projects && data.projects.length > 0) {
                var projectSelectionMessage = 'Please enter the number of the project related to the project name you would like for the Internal Audit:\n';
                data.projects.forEach((project, index) => {
                    projectSelectionMessage += (index + 1) + '. ' + project.project_name + ' (ID: ' + project.project_id + ')\n';
                });

                var selectedProjectNumber = prompt(projectSelectionMessage);
                var selectedProjectIndex = parseInt(selectedProjectNumber, 10) - 1;

                if (selectedProjectIndex >= 0 && selectedProjectIndex < data.projects.length) {
                    var selectedProject = data.projects[selectedProjectIndex];
                    checkIARecord(selectedProject.project_id, selectedProject.project_name);
                } else {
                    alert('Invalid project number');
                }
            } else {
                alert('No projects found for this customer.');
            }
        })
        .catch(error => {
            console.error('There has been a problem with your fetch operation:', error);
            alert('Error fetching projects');
        });
}

function checkIARecord(projectId, projectName) {
    fetch(`/get-ia-records?project_id=${projectId}&project_name=${encodeURIComponent(projectName)}`)
    .then(response => response.json())
    .then(data => {
        if (data.recordFound) {
            window.location.href = `/internal-audit?project_id=${projectId}&project_name=${encodeURIComponent(projectName)}`;
        } else {
            window.location.href = `/add-ia-record?project_id=${projectId}&project_name=${encodeURIComponent(projectName)}`;
        }
    })
    .catch(error => {
        console.error('Error checking IA record:', error);
    });
}

function viewComplianceProjects() {
    // Redirect to the /view-compliance-projects endpoint
    window.location.href = '/view-compliance-projects';
}
function populateComplianceProjectsTable(projectsData) {
    const tableBody = document.getElementById('compliance-projects-table').querySelector('tbody');
    tableBody.innerHTML = '';

    projectsData.forEach(project => {
        let row = tableBody.insertRow();
        row.insertCell().textContent = project.projectId;
        row.insertCell().textContent = project.projectName;
        row.insertCell().textContent = project.complianceType;
        row.insertCell().textContent = project.startDate;
        row.insertCell().textContent = project.endDate || 'N/A';
        row.insertCell().textContent = project.status;
        // Add more cells or buttons as needed
    });
}
function setupTreeButtons() {
    // Initialize trees only once
    setupTreeContainer(document.getElementById("documentsTreeContainer"), ["ISMS", "Policies"]);
    setupTreeContainer(document.getElementById("recordsTreeContainer"), ["IA", "RA + RTP", "Management Review", "SOA"]);

    document.getElementById("documents").addEventListener("click", function() {
        toggleTree("documentsTreeContainer", "documents");
    });

    document.getElementById("records").addEventListener("click", function() {
        toggleTree("recordsTreeContainer", "records");
    });

    // Adding an event listeners to all the Button from the "Records" tree
    const IAButton = document.querySelector("#recordsTreeContainer .tree-button:nth-of-type(1)");
    if (IAButton) {
        IAButton.addEventListener("click", function() {
            console.log("Current Compliance Type:", currentClientCompliances);
            console.log("Project ID:", projectId);
            openIA();
        });
    } else {
        console.error("IA button not found");
    }

    // Adding an event listeners to all the Button from the "Records" tree
    const RARTPButton = document.querySelector("#recordsTreeContainer .tree-button:nth-of-type(2)");
    if (RARTPButton) {
        RARTPButton.addEventListener("click", function() {
            openRAandRTP();
        });
    } else {
        console.error("RA+RTP button not found");
    }

    const managementReviewButton = document.querySelector("#recordsTreeContainer .tree-button:nth-of-type(3)");
    if (managementReviewButton) {
        managementReviewButton.addEventListener("click", function() {
            openmr();
        });
    } else {
        console.error("Management Review button not found");
    }

    const soaButton = document.querySelector("#recordsTreeContainer .tree-button:nth-of-type(4)");
    if (soaButton) {
        soaButton.addEventListener("click", function() {
            console.log("Current Compliance Type:", currentClientCompliances);
            console.log("Project ID:", projectId);
            opensoa(currentClientCompliances);
        });
    } else {
        console.error("SOA button not found");
    }
}
function openRAandRTP() {
    var apiUrl = '/fetch-customer-projects?customer_id=' + encodeURIComponent(customerId);

    fetch(apiUrl)
    .then(response => response.json())
    .then(data => {
        if (data.projects && data.projects.length === 1) {
            var selectedProject = data.projects[0];
            checkRAandRTPRecord(selectedProject.project_id, selectedProject.project_name);
        } else if (data.projects && data.projects.length > 0) {
            var projectSelectionMessage = 'Please select the project for the RA&RTP:\n';
            data.projects.forEach((project, index) => {
                projectSelectionMessage += (index + 1) + '. ' + project.project_name + ' (ID: ' + project.project_id + ')\n';
            });

            var selectedProjectNumber = prompt(projectSelectionMessage);
            var selectedProjectIndex = parseInt(selectedProjectNumber, 10) - 1;

            if (selectedProjectIndex >= 0 && selectedProjectIndex < data.projects.length) {
                var selectedProject = data.projects[selectedProjectIndex];
                checkRAandRTPRecord(selectedProject.project_id, selectedProject.project_name);
            } else {
                alert('Invalid project number');
            }
        } else {
            alert('No projects found for this customer.');
        }
    })
    .catch(error => {
        console.error('Error fetching projects:', error);
        alert('Error fetching projects');
    });
}

function checkRAandRTPRecord(projectId, projectName) {
    fetch(`/fetch-ra-rtp-data?projectId=${projectId}`)
    .then(response => response.json())
    .then(data => {
        if (!data.id) { // Assuming 'id' is a field indicating a record exists
            var createNewRecord = confirm('No RA&RTP record found for this project. Would you like to create a new RA&RTP record?');
            if (createNewRecord) {
                // Submit the new RA-RTP record
                const raRtpData = {
                    project_id: projectId,
                    project_name: projectName,
                    customer_name: clientname,
                    created_by: userfullname,
                    version_number: 1.0 // Initial version number
                };

                fetch('/submit-ra-rtp', {
                    method: 'POST',
                    headers: {'Content-Type': 'application/json'},
                    body: JSON.stringify(raRtpData)
                })
                .then(response => {
                    if (response.ok) {
                        alert('RA-RTP record created successfully.');
                        window.location.href = `/risk-assessment-treatment/${projectId}`; // Redirect to the RA-RTP page
                    } else {
                        throw new Error('Failed to create RA-RTP record.');
                    }
                });
            }
        } else {
            window.location.href = `/risk-assessment-treatment/${projectId}`; // Redirect to the RA-RTP page
        }
    })
    .catch(error => {
        console.error('Error checking RA&RTP record:', error);
        alert('Error checking RA&RTP record');
    });
}


function setupTreeContainer(treeContainer, buttonIds) {
    buttonIds.forEach(id => {
        var button = createTreeButton(id);
        treeContainer.appendChild(button);
    });
    treeContainer.setAttribute("data-initialized", "true");
}
function toggleTree(treeContainerId, triggerButtonId) {
    closeAllTrees(); // Close all trees first

    var treeContainer = document.getElementById(treeContainerId);
    var dashboard = document.querySelector('.dashboard');
    var buttonPos = document.getElementById(triggerButtonId).getBoundingClientRect();

    treeContainer.style.top = buttonPos.top + window.scrollY + "px";
    treeContainer.style.left = (buttonPos.right + 10) + "px";

    if (treeContainer.style.display === "none") {
        treeContainer.style.display = "block";
        // Expand the menu wrap by adjusting the dashboard grid layout
        dashboard.style.gridTemplateColumns = "250px auto"; // Increase first value as needed
    } else {
        treeContainer.style.display = "none";
        // Reset the dashboard grid layout
        dashboard.style.gridTemplateColumns = "400px auto"; // Original size
    }
}
function createTreeButton(text) {
    var button = document.createElement("button");
    button.className = "tree-button";
    button.innerText = text;
    return button;
}
function closeAllTrees() {
    var dropdowns = document.querySelectorAll('.tree-container');
    dropdowns.forEach(function(dropdown) {
        dropdown.style.display = 'none';
    });
}

function openmr() {
    var apiUrl = '/fetch-customer-projects?customer_id=' + encodeURIComponent(customerId);

    fetch(apiUrl)
    .then(response => {
        if (!response.ok) {
            throw new Error('Network response was not ok');
        }
        return response.json();
    })
    .then(data => {
        if (data.projects && data.projects.length > 0) {
            if (data.projects.length === 1) {
                // If there's only one project, redirect immediately without prompting
                var singleProject = data.projects[0];
                window.location.href = '/management-review/' + encodeURIComponent(singleProject.project_id);
            } else {
                // More than one project, prompt the user to choose
                var projectSelectionMessage = 'Please enter the number of the project you would like to review:\n';
                data.projects.forEach((project, index) => {
                    projectSelectionMessage += (index + 1) + '. ' + project.project_name + ' (ID: ' + project.project_id + ')\n';
                });

                var selectedProjectNumber = prompt(projectSelectionMessage);
                var selectedProjectIndex = parseInt(selectedProjectNumber, 10) - 1;

                if (selectedProjectIndex >= 0 && selectedProjectIndex < data.projects.length) {
                    var selectedProject = data.projects[selectedProjectIndex];
                    var projectId = selectedProject.project_id;

                    // Redirect to the management-review page with projectId
                    window.location.href = '/management-review/' + encodeURIComponent(projectId);
                } else {
                    alert('Invalid project number');
                }
            }
        } else {
            alert('No management review projects found.');
        }
    })
    .catch(error => {
        console.error('There has been a problem with your fetch operation:', error);
        alert('Error fetching management review projects');
    });
}
