var projectId, complianceType;
var controlStatus, notes, links, customer_id;
var globalData ;
let current_chapter_ids = [];

document.addEventListener('DOMContentLoaded', function() {
    document.getElementById('statusFilter').addEventListener('change', function() {
        const selectedStatus = this.value;
        console.log('The selected status is:', selectedStatus);
        console.log('The current_chapter_ids are:', current_chapter_ids);

        if (selectedStatus === "All") {
            fetchAndDisplayGapAnalysis(complianceType);
        } else if (selectedStatus === "Not Answered") {
            updateUnansweredControls(projectId,current_chapter_ids);
        } else {
            updateComplianceData(projectId, selectedStatus, current_chapter_ids);
        }
    });

    fetch('/get-gap-analysis-info')
        .then(response => {
            if (!response.ok) {
                throw new Error('Failed to load project details');
            }
            return response.json();
        })
        .then(info => {
            projectId = info.projectId;
            complianceType = info.complianceType;

            // Check if complianceType is valid before making another fetch call
            if (!complianceType) {
                throw new Error('Compliance type is null or undefined');
            }

            fetchAndDisplayGapAnalysis(complianceType);
        })
        .catch(error => {
            console.error('Error:', error);
            // Handle what to do if complianceType is not available
            alert('Unable to load compliance type information.');
        });
});


function fetchAndDisplayGapAnalysis(complianceType) {
    if (!complianceType) {
        console.error('Compliance type is null or undefined at the time of API call.');
        return; // Prevent making an API call with a null/undefined compliance type
    }

    fetch(`/show-gap-analysis-by-type?compliance_type=${encodeURIComponent(complianceType)}`)
        .then(response => {
            if (!response.ok) {
                throw new Error('Network response was not ok');
            }
            return response.json();
        })
        .then(data => {
            populateTabs(data);
        })
        .catch(error => console.error('Error:', error));
}

function populateTabs(data) {
    const chapters = new Set(data.map(item => item.chapter_name));
    const tabContainer = document.getElementById('tabContainer');

    // First, clear existing tabs if any (optional)
    tabContainer.innerHTML = '';

    chapters.forEach(chapter => {
        const tab = document.createElement('button');
        tab.textContent = chapter;
        tab.className = 'tab'; // Added a common class name for CSS styling if needed

        // Set initial active tab, optionally
        if (tabContainer.children.length === 0) {
            tab.classList.add('active-tab');
            loadTableData(data, chapter);
            CurrentChapterName = chapter;
        }

        tab.onclick = () => {
            // Remove active class from all tabs
            document.querySelectorAll('#tabContainer .tab').forEach(t => t.classList.remove('active-tab'));

            // Add active class to the current tab
            tab.classList.add('active-tab');

            // Load data for clicked chapter
            loadTableData(data, chapter);
        };

        tabContainer.appendChild(tab);
    });
}

function loadTableData(data, chapterName) {
    const tableBody = document.getElementById('gapAnalysisTable').getElementsByTagName('tbody')[0];
    tableBody.innerHTML = ''; // Clear existing data
    globalData = data;

    // Ensure the array is cleared when loading new data for a different chapter
    current_chapter_ids = [];

    // Filter data based on chapterName and populate the table
    data.filter(rowData => rowData.chapter_name === chapterName)
        .forEach(rowData => {
            const row = tableBody.insertRow();
            current_chapter_ids.push(rowData.control_id); // Add the control_id to the array

            // Define the order and data for each cell
            const cellData = [
                rowData.control_id,      // Control ID
                rowData.chapter_name,    // Chapter Name
                rowData.control_text,    // Control Text
                rowData.requirements,    // Requirements
                rowData.explanation      // Explanation
            ];

            // Populate each cell with the respective data
            cellData.forEach(text => {
                const cell = row.insertCell();
                cell.textContent = text;
            });

            // Add click event to open a new row for details
            row.addEventListener('click', function() {
                openDetailRow(row, rowData);
            });
        });
}

async function updateComplianceData(projectId, Status, idsarray) {
    const tableBody = document.getElementById('gapAnalysisTable').getElementsByTagName('tbody')[0];
    tableBody.innerHTML = ''; // Clear existing data before adding new data

    idsarray.forEach(id => {
        const url = `/get-compliance-info?projectId=${encodeURIComponent(projectId)}&control_id=${encodeURIComponent(id)}`;
        fetch(url)
            .then(response => {
                if (response.ok) {
                    return response.json();
                }
            })
            .then(data => {
                if (data.useGlobalData || data.control_status === Status) {
                    // Use globalData if the control_status matches or we are directed to use it from HTTP 248
                    const globalDataEntry = globalData.find(item => item.control_id === id);
                    if (globalDataEntry) {
                        updateTableWithGlobalData(globalDataEntry);
                    }
                }
            })
            .catch(error => {
                console.error('Error fetching compliance info:', error);
                updateTableWithFetchError(id);
            });
    });

    function updateTableWithGlobalData(rowData) {
        const row = tableBody.insertRow();
        const cellData = [
            rowData.control_id,      // Control ID
            rowData.chapter_name,    // Chapter Name
            rowData.control_text,    // Control Text
            rowData.requirements,    // Requirements
            rowData.explanation      // Explanation
        ];
        cellData.forEach(text => {
            const cell = row.insertCell();
            cell.textContent = text;
        });
        row.addEventListener('click', function() {
            openDetailRow(row, rowData);
        });
    }
}

async function updateUnansweredControls(projectId, idsarray) {
    const tableBody = document.getElementById('gapAnalysisTable').getElementsByTagName('tbody')[0];
    tableBody.innerHTML = ''; // Clear existing data before adding new data

    for (const id of idsarray) {
        try {
            const url = `/get-compliance-info?projectId=${encodeURIComponent(projectId)}&control_id=${encodeURIComponent(id)}`;
            const response = await fetch(url);
            if (response.status === 248) {  // Check for HTTP 248 status
                const controlData = globalData.find(item => item.control_id === id);
                if (controlData) {
                    updateTableWithGlobalData(controlData);
                } else {
                    updateTableWithNoData(id); // Handle cases where there is no data at all
                }
            }
        } catch (error) {
            console.error('Error fetching control status:', error);
            updateTableWithFetchError(id);
        }
    }

    function updateTableWithGlobalData(rowData) {
        const row = tableBody.insertRow();
        const cellData = [
            rowData.control_id,      // Control ID
            rowData.chapter_name,    // Chapter Name
            rowData.control_text,    // Control Text
            rowData.requirements,    // Requirements
            rowData.explanation      // Explanation
        ];
        cellData.forEach(text => {
            const cell = row.insertCell();
            cell.textContent = text;
        });
        row.addEventListener('click', function() {
            openDetailRow(row, rowData);
        });
    }

    function updateTableWithNoData(controlId) {
        const row = tableBody.insertRow();
        const cell = row.insertCell();
        cell.textContent = `No data available for Control ID: ${controlId}`;
        cell.colSpan = 5;  // Assuming there are 5 columns as per the data structure
    }

    function updateTableWithFetchError(controlId) {
        const row = tableBody.insertRow();
        const cell = row.insertCell();
        cell.textContent = `Error fetching data for Control ID: ${controlId}`;
        cell.colSpan = 5;  // Cover all columns with the error message
    }
}

function openDetailRow(clickedRow, rowData) {
    removeDetailRows();

    if (rowData && rowData.control_id) {
        // Calculate the index for the new detail row
        const detailRowIndex = Array.from(clickedRow.parentNode.rows).indexOf(clickedRow) + 1;

        // Insert a new detail row at the calculated index
        const detailRow = clickedRow.parentNode.insertRow(detailRowIndex);
        detailRow.className = 'detail-row';

        // Create a cell that spans all columns
        const cell = detailRow.insertCell();
        cell.colSpan = clickedRow.cells.length;

        // Populate the detail cell
        populateDetailCell(cell, rowData);

        getComplianceInfo(projectId, rowData.control_id, rowData);
    }
}

function populateDetailCell(cell, rowData) {
    const detailTable = document.createElement('table');
    detailTable.className = 'detail-table';

    // Add rows for Control Status, Notes, and Links
    const statusRow = detailTable.insertRow();
    statusRow.innerHTML = `<th>Control ${rowData.control_id} Status</th><td><select id="controlStatus-${rowData.control_id}"><option value="">Please Choose a Suitable Value</option><option value="Compliant">Compliant</option><option value="Partially Compliant">Partially Compliant</option><option value="Non-Compliant">Non-Compliant</option><option value="Not Applicable">Not Applicable</option></select></td>`;

    const notesRow = detailTable.insertRow();
    notesRow.innerHTML = `<th>Notes From User Response</th><td><textarea id="userNotes-${rowData.control_id}"></textarea></td>`;

    const linksRow = detailTable.insertRow();
    linksRow.innerHTML = `<th>Links</th><td><input type="text" id="links-${rowData.control_id}" /></td>`;

    // Modified Non-Conformity row with red text and white background
    const nonConformityRow = detailTable.insertRow();
    nonConformityRow.innerHTML = `
        <th class="button3" style="background-color: white; color: red;">Non-Conformity</th>
        <td>
            <select id="nonConformity-${rowData.control_id}" class="detail-select" style="background-color: white; color: red;">
                <option value="">Please Choose a Suitable Value</option>
                <!-- Additional options will be added here -->
            </select>
        </td>`;
    fetchNonConformityOptions(rowData.control_id, nonConformityRow.querySelector('select'));

    // Modified Opportunity for Improvement row with blue text and white background
    const opportunityRow = detailTable.insertRow();
    opportunityRow.innerHTML = `
        <th class="button2" style="background-color: white; color: blue;">Opportunity for Improvement</th>
        <td>
            <select id="opportunity-${rowData.control_id}" class="detail-select" style="background-color: white; color: blue;">
                <option value="">Please Choose a Suitable Value</option>
                <!-- Additional options will be added here -->
            </select>
        </td>`;
    fetchOpportunityOptions(rowData.control_id, opportunityRow.querySelector('select'));

    // Set the initial values based on the received data
    const controlStatusSelect = statusRow.querySelector('select');
    const userNotesTextarea = notesRow.querySelector('textarea');
    const linksInput = linksRow.querySelector('input[type="text"]');

    if (rowData.control_status) {
        controlStatusSelect.value = rowData.control_status;
    }
    if (rowData.notes) {
        userNotesTextarea.value = rowData.notes;
    }
    if (rowData.links) {
        linksInput.value = rowData.links;
    }

    // Add a Save button row
    const saveRow = detailTable.insertRow();
    saveRow.innerHTML = `<td colspan="2"><button onclick="Save()" title="Save">Save Changes</button></td>`;

    // Append the detail table to the cell
    cell.appendChild(detailTable);
}

function fetchNonConformityOptions(controlId, selectElement, callback) {
    fetch(`/get-gap-analysis-options/${controlId}`)
        .then(response => response.json())
        .then(data => {
            data.nonConformityOptions.forEach(option => {
                const opt = document.createElement('option');
                opt.value = option;
                opt.textContent = option;
                selectElement.appendChild(opt);
            });
            if (callback) callback(); // Call the callback function after options are added
        })
        .catch(error => console.error('Error fetching Non-Conformity options:', error));
}

function fetchOpportunityOptions(controlId, selectElement, callback) {
    fetch(`/get-gap-analysis-options/${controlId}`)
        .then(response => response.json())
        .then(data => {
            // Assuming data.opportunityOptions contains the options for the dropdown
            data.opportunityOptions.forEach(option => {
                const opt = document.createElement('option');
                opt.value = option; // Set the value of the option element
                opt.textContent = option; // Set the display text of the option element
                selectElement.appendChild(opt); // Add the option element to the select dropdown
            });
            if (callback) callback(); // Execute the callback function after options have been added
        })
        .catch(error => console.error('Error fetching Opportunity options:', error));
}

function populateDropdown(selectId, options) {
    const select = document.getElementById(selectId);
    options.forEach(option => {
        const opt = document.createElement('option');
        opt.value = option;
        opt.textContent = option;
        select.appendChild(opt);
    });
}

function removeDetailRows() {
    const detailRows = document.querySelectorAll('.detail-row');
    detailRows.forEach(row => row.remove());
}

function Save() {
    const controlData = [];

    document.querySelectorAll('.detail-row').forEach(detailRow => {
        const control_id = detailRow.querySelector('select').id.split('-')[1].replace(/\./g, '.'); // Keep the original format

        const control_status_select = document.getElementById(`controlStatus-${control_id}`);
        const notes_textarea = document.getElementById(`userNotes-${control_id}`);
        const links_input = document.getElementById(`links-${control_id}`);
        const nonConformity_select = document.getElementById(`nonConformity-${control_id}`);
        const opportunity_select = document.getElementById(`opportunity-${control_id}`);

        console.log({ control_status_select, notes_textarea, links_input, nonConformity_select, opportunity_select });

        if (control_status_select && notes_textarea && links_input && nonConformity_select && opportunity_select) {
            controlData.push({
                control_id,
                control_status: control_status_select.value,
                notes: notes_textarea.value,
                links: links_input.value,
                nonConformity: nonConformity_select.value,
                opportunity: opportunity_select.value
            });
        } else {
            console.error('One or more elements could not be found for control_id:', control_id);
        }
    });

    if (!controlData.length) {
        console.error('No control data to save.');
        return; // Early exit if no data to send
    }

    fetch(`/update-compliance-project/${projectId}`, {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify({ controlData })
    })
    .then(response => {
        if (!response.ok) {
            return response.json().then(data => {
                throw new Error(data.error || 'Unknown error');
            });
        }
        return response.json();
    })
    .then(data => {
        console.log(data.message); // 'Data saved successfully' or any message returned by server
    })
    .catch(error => {
        console.error('Error:', error);
    });
}

function getComplianceInfo(projectId, control_id, rowData) {
    const url = `/get-compliance-info?projectId=${encodeURIComponent(projectId)}&control_id=${encodeURIComponent(control_id)}`;

    fetch(url)
        .then(response => {
            if (!response.ok) {
                if (response.status === 404) {
                    console.log(`No data found for control_id: ${control_id}`);
                    clearDetailRowData(control_id);
                    return null;
                } else if (response.status === 400) {
                    console.error('Bad Request: Check the request parameters.');
                    return null;
                }
                throw new Error(`Network response was not ok, status: ${response.status}`);
            }
            return response.json();
        })
        .then(data => {
            if (!data) return; // Stop processing if no data

            console.log('Response Data:', data);
            updateDetailRowWithData(control_id, data);
        })
        .catch(error => {
            console.error('Error fetching compliance info:', error);
        });
}


function clearDetailRowData(control_id) {
    const controlStatusSelect = document.getElementById(`controlStatus-${control_id}`);
    const userNotesTextarea = document.getElementById(`userNotes-${control_id}`);
    const linksInput = document.getElementById(`links-${control_id}`);

    if (controlStatusSelect) controlStatusSelect.value = '';
    if (userNotesTextarea) userNotesTextarea.value = '';
    if (linksInput) linksInput.value = '';
}

function updateDetailRowWithData(control_id, data) {
    const controlStatusSelect = document.getElementById(`controlStatus-${control_id}`);
    const userNotesTextarea = document.getElementById(`userNotes-${control_id}`);
    const linksInput = document.getElementById(`links-${control_id}`);
    const nonConformitySelect = document.getElementById(`nonConformity-${control_id}`);
    const opportunitySelect = document.getElementById(`opportunity-${control_id}`);


    if (controlStatusSelect && data.control_status) {
        controlStatusSelect.value = data.control_status;
    }
    if (userNotesTextarea && data.notes) {
        userNotesTextarea.value = data.notes;
    }
    if (linksInput && data.links) {
        linksInput.value = data.links;
    }
    if (nonConformitySelect && data.non_conformity) {
        nonConformitySelect.value = data.non_conformity;
    }
    if (opportunitySelect && data.opportunity_for_improvement) {
        opportunitySelect.value = data.opportunity_for_improvement;
    }
}