var user_id;
var customer_id;
var cmmiValue;

document.addEventListener('DOMContentLoaded', () => {
    initializeCharts();
    fetchUserIdFromSession();

    // Setup event listeners for static and dynamically added dropdown buttons
    document.addEventListener('click', function(event) {
        const target = event.target;
        const dropBtn = target.closest('.dropbtn');

        if (dropBtn) {
            // Prevent the click event from closing the dropdown immediately
            event.stopPropagation();

            // Find the closest parent that has either 'company-name' or 'user-info' class
            const parentContainer = dropBtn.closest('.company-name, .user-info');

            // Toggle the dropdown content within this container
            const dropdownContent = parentContainer.querySelector('.dropdown-content');
            if (dropdownContent) {
                dropdownContent.classList.toggle('show');
            }
        } else {
            // Clicked outside, close all dropdowns
            closeAllDropdowns();
        }
    });

    // Close dropdowns when clicking outside
    document.body.addEventListener('click', closeAllDropdowns);

    // Setup event listener for the "Risk Assessment"  link
    const riskAssessmentLink = document.querySelector('.sidebar-link[href="#risk-assessment"]');
    if (riskAssessmentLink) {
        riskAssessmentLink.addEventListener('click', function(event) {
            event.preventDefault(); // Prevent the default navigation
            openRAandRTP();
        });
    }
    // Setup event listener for the "Gap Analysis" link
    const gapAnalysisLink = document.querySelector('.sidebar-link[href="#Gap-Analysis"]');
    if (gapAnalysisLink) {
        gapAnalysisLink.addEventListener('click', function(event) {
            event.preventDefault();
            openGapAnalysisDoc();
        });
    }
    // For Statement of Applicability (SOA)
    const soaLink = document.querySelector('.sidebar-link[href="#SOA"]');
    if (soaLink) {
        soaLink.addEventListener('click', function(event) {
            event.preventDefault();
            opensoa();
        });
    }

    // For Internal Audit
    const internalAuditLink = document.querySelector('.sidebar-link[href="#internal-audit"]');
    if (internalAuditLink) {
        internalAuditLink.addEventListener('click', function(event) {
            event.preventDefault();
            openIA();
        });
    }

    // For Goals & KPI's
    const goalsKPIsLink = document.querySelector('.sidebar-link[href="#goals_kpis"]');
    if (goalsKPIsLink) {
        goalsKPIsLink.addEventListener('click', function(event) {
            event.preventDefault();
            openGoalsKPIsDoc();
        });
    }

    // For Management Review
    const managementReviewLink = document.querySelector('.sidebar-link[href="#management-review"]');
    if (managementReviewLink) {
        managementReviewLink.addEventListener('click', function(event) {
            event.preventDefault();
            openmr();
        });
    }
});

function closeAllDropdowns() {
    document.querySelectorAll('.dropdown-content.show').forEach(dropdownContent => {
        dropdownContent.classList.remove('show');
    });
}

function initializeCharts() {
    // Placeholder data - replace with your actual data
    const projectProgressData = [30, 30, 30];
    const controlsDistributionData = [4, 3, 2, 1];

    // Project Progress Pie Chart
    const projectProgressCtx = document.getElementById('ProjectsStatus').getContext('2d');
    const projectProgressChart = new Chart(ProjectsStatus, {
        type: 'pie',
        data: {
            labels: ['Finished', 'In Progress', 'Not Started'],
            datasets: [{
                data: projectProgressData,
                backgroundColor: ['#228B22', '#FDDA0D', '#C41E3A']
            }]
        },
          options: {
            responsive: true,
            maintainAspectRatio: true,
            plugins: {
                title: {
                    display: true,
                    text: 'Projects Progress Distribution',
                    font: {
                    size: 20
                    },
                }
            }
          }
        });

    // Controls Distribution Pie Chart
    const controlsDistributionCtx = document.getElementById('controlsDistributionChart').getContext('2d');
    const controlsDistributionChart = new Chart(controlsDistributionCtx, {
        type: 'pie',
        data: {
            labels: ['Less Then 1 Month', 'Less Then 2 Months', 'Less Then 3 Months', 'Over 3 Months'],
            datasets: [{
                data: controlsDistributionData,
                backgroundColor: ['#228B22', '#FDDA0D', '#FFAA33', '#C41E3A']
            }]
        },
          options: {
            responsive: true,
            maintainAspectRatio: true,
            plugins: {
                title: {
                    display: true,
                    text: 'Projects By Time',
                    font: {
                    size: 20
                    },
                }
            }
          }
        });
}

function fetchUserIdFromSession() {
    // Make a GET request to a server endpoint that returns the session data
    fetch('/get-user-id', {
        method: 'GET',
        credentials: 'include',
    })
    .then(response => response.json())
    .then(data => {
        // Check if 'user_id' is available in the session data
        if (data.user_id !== undefined) {
            user_id = data.user_id;
            console.log('User ID:', user_id);

            // Fetch user data using 'user_id'
            fetchUserData(user_id);
        } else {
            console.error('User is not authenticated or session data is missing.');
        }
    })
    .catch(error => console.error('Error fetching user session data:', error));
}

function fetchUserData(user_Id) {
    const apiUrl = `/api/user-data/${user_Id}`;

    // Make a GET request to the API endpoint
    fetch(apiUrl)
        .then(response => {
            if (!response.ok) {
                throw new Error('Network response was not ok');
            }
            return response.json();
        })
        .then(data => {
            customer_id = data.customer_id;
            // Creating a UserData object with the response data
            const userData = {
                first_name: data.first_name,
                last_name: data.last_name,
                last_login: data.last_login,
                customer_id: data.customer_id,
            };
            updateUserUI(userData);
            console.log(userData);
            const customerIds = getCustomerIds(data.customer_id);
            console.log('populateCompanyDropdown called with customerIds:', customerIds);
            populateCompanyDropdown(customerIds);
        })
        .catch(error => {
            // Handle errors that occurred during the fetch request
            console.error('Fetch error:', error);
        });
}

function getCustomerIds(customerIdString) {
    // Check if customerIdString is not undefined or null
    if (customerIdString) {
        // Split the string by commas to create an array
        return customerIdString.split(",");
    } else {
        // Return an empty array if customerIdString is undefined or null
        return [];
    }
}

function updateUserUI(userData) {
    // Update the UI with the user data
    document.getElementById('firstName').textContent = userData.first_name;
    document.getElementById('lastName').textContent = userData.last_name;

}

async function populateCompanyDropdown(customerIds) {
    const companyListContainer = document.querySelector('#companyListContainer');
    if (!companyListContainer) {
        console.error('The company list container element was not found.');
        return;
    }

    companyListContainer.innerHTML = ''; // Clear previous content

    if (customerIds.length > 1) {
        const companyDiv = document.createElement('div');
        companyDiv.className = 'company-name';

        const button = document.createElement('button');
        button.className = 'dropbtn';
        button.innerHTML = `Clients <i class="fas fa-chevron-down"></i>`;
        button.addEventListener('click', function() {
            this.nextElementSibling.classList.toggle('show');
        });

        const dropdownContent = document.createElement('div');
        dropdownContent.className = 'dropdown-content';

        companyDiv.appendChild(button);
        companyDiv.appendChild(dropdownContent);
        companyListContainer.appendChild(companyDiv);

        // Fetch and populate for each customerId
        for (const customerId of customerIds) {
            await fetchAndPopulate(customerId, dropdownContent);
        }
    } else if (customerIds.length === 1) {
        // Only one customerId, fetch and display its name directly
        await fetchAndPopulate(customerIds[0], companyListContainer, true);
    }

    // Click outside to close dropdowns
    window.onclick = function(event) {
        if (!event.target.matches('.dropbtn')) {
            var dropdowns = document.getElementsByClassName("dropdown-content");
            for (var i = 0; i < dropdowns.length; i++) {
                var openDropdown = dropdowns[i];
                if (openDropdown.classList.contains('show')) {
                    openDropdown.classList.remove('show');
                }
            }
        }
    };
}

async function fetchAndPopulate(customerId, container, isSingle = false) {
    const apiUrl = `/fetch-customer-projects?customer_id=${customerId}`;
    try {
        const response = await fetch(apiUrl);
        if (!response.ok) {
            throw new Error(`Error fetching projects for customer ID ${customerId}: ${response.statusText}`);
        }
        const data = await response.json();

        if (isSingle) {
            const companyName = data.projects[0]?.customer_name || 'Unknown';
            const companyDiv = document.createElement('div');
            companyDiv.className = 'company-name';

            const button = document.createElement('button');
            button.className = 'dropbtn';
            button.innerHTML = `${companyName} <i class="fas fa-chevron-down"></i>`;
            button.addEventListener('click', function() {
                this.nextElementSibling.classList.toggle('show');
            });

            const dropdownContent = document.createElement('div');
            dropdownContent.className = 'dropdown-content';

            companyDiv.appendChild(button);
            companyDiv.appendChild(dropdownContent);
            container.appendChild(companyDiv);
        } else {
            data.projects.forEach(project => {
                const clientName = project.customer_name || 'Unknown';
                const projectElement = document.createElement('a');
                projectElement.textContent = clientName;
                container.appendChild(projectElement);
            });
        }
    } catch (error) {
        console.error("Fetch error:", error);
    }
}

function openRAandRTP() {
    var apiUrl = '/fetch-customer-projects';

    fetch(apiUrl)
    .then(response => response.json())
    .then(data => {
        if (data.projects && data.projects.length === 1) {
            var selectedProject = data.projects[0];
            checkRAandRTPRecord(selectedProject.project_id, selectedProject.project_name);
        } else if (data.projects && data.projects.length > 0) {
            var projectSelectionMessage = 'Please select the project for the RA&RTP:\n';
            data.projects.forEach((project, index) => {
                projectSelectionMessage += `${index + 1}. ${project.customer_name} - ${project.project_name} (ID: ${project.project_id})\n`;
            });

            var selectedProjectNumber = prompt(projectSelectionMessage);
            var selectedProjectIndex = parseInt(selectedProjectNumber, 10) - 1;

            if (selectedProjectIndex >= 0 && selectedProjectIndex < data.projects.length) {
                var selectedProject = data.projects[selectedProjectIndex];
                checkRAandRTPRecord(selectedProject.project_id, selectedProject.project_name);
            } else {
                alert('Invalid project number');
            }
        } else {
            alert('No projects found for this customer.');
        }
    })
    .catch(error => {
        console.error('Error fetching projects:', error);
        alert('Error fetching projects');
    });
}


function openGapAnalysisDoc() {
    fetch(`/api/compliance-projects`)
        .then(response => response.json())
        .then(data => {
            if (data.length === 1) {
                submitGapAnalysisRequest(data[0].projectId, data[0].complianceType);
            } else if (data && data.length > 0) {
                const projectNames = data.map((project, index) => `${index + 1}. ${project.customerName} - ${project.projectName}`);
                const projectSelectionPrompt = projectNames.join('\n');
                const selectedProjectNumber = prompt(`Please select a project:\n${projectSelectionPrompt}`);

                if (selectedProjectNumber) {
                    const selectedProjectIndex = parseInt(selectedProjectNumber, 10) - 1;
                    if (selectedProjectIndex >= 0 && selectedProjectIndex < data.length) {
                        const selectedProject = data[selectedProjectIndex];
                        submitGapAnalysisRequest(selectedProject.projectId, selectedProject.complianceType);
                    } else {
                        alert('Invalid selection. Please try again.');
                    }
                }
            } else {
                alert('No projects found for this customer.');
            }
        })
        .catch(error => {
            console.error('Error fetching compliance projects:', error);
            alert('An error occurred while fetching compliance projects.');
        });
}

function submitGapAnalysisRequest(projectId, complianceType) {
    fetch('/gap-analysis-doc', {
        method: 'POST',
        headers: {
            'Content-Type': 'application/x-www-form-urlencoded',
        },
        body: `projectId=${encodeURIComponent(projectId)}&complianceType=${encodeURIComponent(complianceType)}`
    }).then(response => {
        if (response.redirected) {
            window.location.href = response.url;  // Follow the redirect URL provided by the server
        } else {
            throw new Error('Expected a redirect but did not receive one.');
        }
    }).catch(error => {
        console.error('Error posting gap analysis request:', error);
        alert('An error occurred while processing your request.');
    });
}


function checkRAandRTPRecord(projectId, projectName) {
    fetch(`/fetch-ra-rtp-data?projectId=${projectId}`)
    .then(response => response.json())
    .then(data => {
        if (!data.id) { // Assuming 'id' is a field indicating a record exists
            var createNewRecord = confirm('No RA&RTP record found for this project. Would you like to create a new RA&RTP record?');
            if (createNewRecord) {
                // Submit the new RA-RTP record
                const raRtpData = {
                    project_id: projectId,
                    project_name: projectName,
                    customer_name: clientname,
                    created_by: userfullname,
                    version_number: 1.0
                };

                fetch('/submit-ra-rtp', {
                    method: 'POST',
                    headers: {'Content-Type': 'application/json'},
                    body: JSON.stringify(raRtpData)
                })
                .then(response => {
                    if (response.ok) {
                        alert('RA-RTP record created successfully.');
                        window.location.href = `/risk-assessment-treatment/${projectId}`; // Redirect to the RA-RTP page
                    } else {
                        throw new Error('Failed to create RA-RTP record.');
                    }
                });
            }
        } else {
            window.location.href = `/risk-assessment-treatment/${projectId}`;
        }
    })
    .catch(error => {
        console.error('Error checking RA&RTP record:', error);
        alert('Error checking RA&RTP record');
    });
}

function opensoa() {
    var apiUrl = '/fetch-customer-projects';

    fetch(apiUrl)
        .then(response => {
            if (!response.ok) {
                throw new Error('Network response was not ok');
            }
            return response.json();
        })
        .then(data => {
            if (data.projects && data.projects.length === 1) {
                // Only one project, so perform the action directly
                var selectedProject = data.projects[0];
                redirectToSOA(selectedProject.project_id, selectedProject.compliance_type);
            } else if (data.projects && data.projects.length > 0) {
                var projectSelectionMessage = 'Please Enter the number of the Project related to the Project name you would like to see the SOA:\n';
                data.projects.forEach((project, index) => {
                    projectSelectionMessage += `${index + 1}. ${project.customer_name} - ${project.project_name} (ID: ${project.project_id})\n`;
                });

                var selectedProjectNumber = prompt(projectSelectionMessage);
                var selectedProjectIndex = parseInt(selectedProjectNumber, 10) - 1;

                if (selectedProjectIndex >= 0 && selectedProjectIndex < data.projects.length) {
                    var selectedProject = data.projects[selectedProjectIndex];
                    redirectToSOA(selectedProject.project_id, selectedProject.compliance_type);
                } else {
                    alert('Invalid project number');
                }
            } else {
                alert('No projects found for this customer.');
            }
        })
        .catch(error => {
            console.error('There has been a problem with your fetch operation:', error);
            alert('Error fetching projects');
        });
}

function redirectToSOA(projectId, complianceType) {
    fetch('/set-SOA-session', {
        method: 'POST',
        headers: {
            'Content-Type': 'application/x-www-form-urlencoded',
        },
        body: `complianceType=${encodeURIComponent(complianceType)}&projectId=${encodeURIComponent(projectId)}`
    }).then(response => {
        if (response.ok) {
            window.location.href = '/SOA';
        } else {
            throw new Error('Failed to set session data');
        }
    }).catch(error => {
        console.error('Error setting session data:', error);
        alert('Failed to initialize session data for SOA');
    });
}

function openIA() {
    var apiUrl = '/fetch-customer-projects';

    fetch(apiUrl)
        .then(response => {
            if (!response.ok) {
                throw new Error('Network response was not ok');
            }
            return response.json();
        })
        .then(data => {
            if (data.projects && data.projects.length === 1) {
                // Only one project, so perform the action directly
                var selectedProject = data.projects[0];
                checkIARecord(selectedProject.project_id, selectedProject.project_name);
            } else if (data.projects && data.projects.length > 0) {
                var projectSelectionMessage = 'Please enter the number of the project related to the project name you would like for the Internal Audit:\n';
                data.projects.forEach((project, index) => {
                    projectSelectionMessage += `${index + 1}. ${project.customer_name} - ${project.project_name} (ID: ${project.project_id})\n`;
                });

                var selectedProjectNumber = prompt(projectSelectionMessage);
                var selectedProjectIndex = parseInt(selectedProjectNumber, 10) - 1;

                if (selectedProjectIndex >= 0 && selectedProjectIndex < data.projects.length) {
                    var selectedProject = data.projects[selectedProjectIndex];
                    checkIARecord(selectedProject.project_id, selectedProject.project_name);
                } else {
                    alert('Invalid project number');
                }
            } else {
                alert('No projects found for this customer.');
            }
        })
        .catch(error => {
            console.error('There has been a problem with your fetch operation:', error);
            alert('Error fetching projects');
        });
}


function checkIARecord(projectId, projectName) {
    fetch(`/get-ia-records?project_id=${projectId}&project_name=${encodeURIComponent(projectName)}`)
    .then(response => {
        if (!response.ok) {
            throw new Error('Network response was not ok');
        }
        return response.json();
    })
    .then(data => {
        // Check if the data array is not empty, which means records exist
        if (data.length > 0) {
            // Store projectId and projectName in sessionStorage
            sessionStorage.setItem('projectId', projectId);
            sessionStorage.setItem('projectName', encodeURIComponent(projectName));
            window.location.href = '/internal-audit';
        } else {
            window.location.href = `/add-ia-record?project_id=${projectId}&project_name=${encodeURIComponent(projectName)}`;
        }
    })
    .catch(error => {
        console.error('Error checking IA record:', error);
        alert('Error checking IA record.');
    });
}

function openmr() {
    var apiUrl = '/fetch-customer-projects';

    fetch(apiUrl)
    .then(response => {
        if (!response.ok) {
            throw new Error('Network response was not ok');
        }
        return response.json();
    })
    .then(data => {
        if (data.projects && data.projects.length > 0) {
            if (data.projects.length === 1) {
                // If there's only one project, redirect immediately without prompting
                var singleProject = data.projects[0];
                window.location.href = '/management-review/' + encodeURIComponent(singleProject.project_id);
            } else {
                // More than one project, prompt the user to choose
                var projectSelectionMessage = 'Please enter the number of the project you would like to review:\n';
                data.projects.forEach((project, index) => {
                    projectSelectionMessage += `${index + 1}. ${project.customer_name} - ${project.project_name} (ID: ${project.project_id})\n`;
                });

                var selectedProjectNumber = prompt(projectSelectionMessage);
                var selectedProjectIndex = parseInt(selectedProjectNumber, 10) - 1;

                if (selectedProjectIndex >= 0 && selectedProjectIndex < data.projects.length) {
                    var selectedProject = data.projects[selectedProjectIndex];
                    var projectId = selectedProject.project_id;

                    // Redirect to the management-review page with projectId
                    window.location.href = '/management-review/' + encodeURIComponent(projectId);
                } else {
                    alert('Invalid project number');
                }
            }
        } else {
            alert('No management review projects found.');
        }
    })
    .catch(error => {
        console.error('There has been a problem with your fetch operation:', error);
        alert('Error fetching management review projects');
    });
}
