var user_id;

document.addEventListener('DOMContentLoaded', function() {
    fetchUserIdFromSession();
    // Remove fetchPastTestsData() if not defined or not needed
});

function fetchUserIdFromSession() {
    fetch('/get-user-id', {
        method: 'GET',
        credentials: 'include',  // Include credentials in the request
    })
    .then(response => response.json())
    .then(data => {
        if (data.user_id !== undefined) {
            user_id = data.user_id;
            console.log('User ID:', user_id);

            // Now that we have the user_id, fetch and display past tests
            fetchAndDisplayPastTests();
            // If fetchUserData() is defined and needed, uncomment the following line
            // fetchUserData(user_id); 
        } else {
            console.error('User is not authenticated or session data is missing.');
        }
    })
    .catch(error => console.error('Error fetching user session data:', error));
}

function fetchAndDisplayPastTests() {
    // Ensure user_id is available before making the request
    if (user_id) {
        fetch(`/api/past-tests-data/${user_id}`)
            .then(response => response.json())
            .then(data => {
                populateTable(data);
            })
            .catch(error => console.error('Error fetching past tests data:', error));
    }
}

function populateTable(testsData) {
    const tableBody = document.getElementById('past-tests-table').querySelector('tbody');
    tableBody.innerHTML = ''; // Clear existing data

    testsData.forEach(test => {
        let row = tableBody.insertRow();
        row.insertCell().textContent = test.testId;
        row.insertCell().textContent = test.score;

        // Format the date string
        let formattedDate = test.date.split('.')[0];
        row.insertCell().textContent = formattedDate;

        // Determine pass or fail based on score
        let status = parseInt(test.score) >= 60 ? "Passed" : "Failed";
        row.insertCell().textContent = status;

        // Add more cells as needed
    });
}
