document.addEventListener('DOMContentLoaded', function() {
    const registrationForm = document.querySelector('.login form');
    var loginForm = document.getElementById('loginForm');
    if (loginForm) {
        loginForm.addEventListener('submit', function(event) {
            event.preventDefault();
            submitLoginForm(loginForm);
        });
    }
    if (registrationForm) {
        registrationForm.addEventListener('submit', function(event) {
            event.preventDefault();
            submitRegistrationForm(registrationForm);
        });
    }
    checkRegistrationSuccess();
});

function submitRegistrationForm(form) {
    const formData = new FormData(form);
    document.getElementById('loadingIndicator').style.display = 'block'; // Show loading indicator

    fetch('/register', {
        method: 'POST',
        body: formData
    })
    .then(response => {
        document.getElementById('loadingIndicator').style.display = 'none'; // Hide loading indicator
        if (!response.ok) throw new Error('Network response was not OK.');
        return response.json();
    })
    .then(data => {
        if (data.error) {
            alert('Error: ' + data.error);
        } else if (data.success) {
            alert('Success: ' + data.message);
            window.location.href = '/login';
        }
    })
    .catch(error => {
        console.error('Error:', error);
        alert("An error occurred. Please try again: " + error.message);
        document.getElementById('loadingIndicator').style.display = 'none'; // Ensure loading indicator is hidden on error
    });
}


function checkRegistrationSuccess() {
    const params = new URLSearchParams(window.location.search);
    if (params.get('registered') === 'true') {
        const messageDiv = document.getElementById('message');
        messageDiv.textContent = "User Successfully Registered - Ask Admin to Be Enabled";
        messageDiv.style.display = 'block';
        params.delete('registered');
        history.pushState({}, '', location.pathname + '?' + params.toString());
    }
}

function submitLoginForm(form) {
    var formData = new FormData(form);
    fetch('/login', {
        method: 'POST',
        body: formData,
        credentials: 'include'
    })
    .then(response => response.json())
    .then(data => {
        if (data.error) {
            alert(data.error);
        } else if (data.passwordChangeRequired) {
            $('#passwordChangeModal').modal('show');
        } else {
            window.location.href = data.redirect;
        }
    })
    .catch(error => {
        console.error('Login failed:', error);
        alert('Login failed, please try again.');
    });
}
